@@ -34,7 +34,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
-#ifdef HAVE_GET_OPT_H
+#ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 #include <stdarg.h>
@@ -249,7 +249,7 @@ static gboolean HB_SIGNON = FALSE;
  * The following is to avoid cl_status sleeping forever. This is due to the 
  * hearbeat's abnormal status or even its crash.  
  */
-unsigned int DEFAULT_TIMEOUT = 2;
+unsigned int DEFAULT_TIMEOUT = 5;
 /* the handler of signal SIGALRM */
 static void quit(int signum);
 
@@ -272,12 +272,9 @@ main(int argc, char ** argv)
 	 * When the program is finished formly, all redundant cl_log clauses 
 	 * will be removed
 	 */ 
-	/* cl_log_enable_stderr(TRUE); */
+	cl_log_enable_stderr(TRUE);
 	cl_log_set_facility(LOG_USER);
 
-	cl_log(LOG_INFO, "start:optind: %d  argv[optind+1]: %s", optind, 
-		argv[optind+1]);
-
 	/*
 	 * To avoid cl_status sleep forever, trigger a timer and dealing with 
 	 * signal SIGALRM. This sleep is due to hearbeat's abnormal status or
@@ -299,7 +296,6 @@ main(int argc, char ** argv)
 				return UNKNOWN_ERROR;
 			}
 
-			/* cl_log(LOG_INFO, "Signing in with heartbeat."); */
 			if (hb->llc_ops->signon(hb, cl_status_name)!= HA_OK) {
 				cl_log(LOG_ERR, "Cannot signon with heartbeat");
 				cl_log(LOG_ERR, "REASON: %s", 
@@ -318,9 +314,6 @@ main(int argc, char ** argv)
 		ret_value = PARAMETER_ERROR;
 	}
 
-	cl_log(LOG_INFO, "End: optind: %d  argv[optind+1]: %s", optind, 
-		argv[optind+1]);
-
 	if (HB_SIGNON == TRUE) {
 		if (hb->llc_ops->signoff(hb) != HA_OK) {
 			cl_log(LOG_ERR, "Cannot sign off from heartbeat.");
@@ -340,7 +333,6 @@ main(int argc, char ** argv)
 		}
 	}
 
-	cl_log(LOG_ERR ,"return value:%d", ret_value);
 	return ret_value;
 }
 
@@ -445,12 +437,10 @@ nodestatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 	};
 
 	if (argc <= optind+1) {
-		fprintf(stderr, "No enough parameter.\n");
+		fprintf(stderr, "Not enough parameters.\n");
 		return PARAMETER_ERROR;
 	}
 
-	cl_log(LOG_INFO, "optind: %d   argv[optindex+1]: %s", optind, 
-		argv[optind+1]);
 	status = hb->llc_ops->node_status(hb, argv[optind+1]);
 	if ( status == NULL ) {
 		fprintf(stderr, "Error. May be due to incorrect node name\n");
@@ -462,8 +452,8 @@ nodestatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		printf("%s\n", status);
 	}
 
-	if ( strncmp(status, "active", 6) == 0 
-	||	strncmp(status, "up", 3) == 0) {
+       if ( strncmp(status, "active", 6) == 0 
+       ||      strncmp(status, "up", 3) == 0) {
 		ret = OK;   /* the node is active */
 	} else {
 		ret = NORMAL_FAIL;  /* the status = "dead" */
@@ -493,8 +483,6 @@ nodetype(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		return PARAMETER_ERROR;
 	}
 
-	cl_log(LOG_INFO, "optind: %d   argv[optindex+1]: %s", optind, 
-		argv[optind+1]);
 	type = hb->llc_ops->node_type(hb, argv[optind+1]);
 	if ( type == NULL ) {
 		fprintf(stderr, "Error. May be due to incorrect node name\n");
@@ -524,9 +512,6 @@ listhblinks(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		return PARAMETER_ERROR;
 	}
 
-	cl_log(LOG_INFO, "optind: %d   argv[optindex+1]: %s", optind, 
-		argv[optind+1]);
-
 	if (hb->llc_ops->init_ifwalk(hb, argv[optind+1]) != HA_OK) {
 		cl_log(LOG_ERR, "Cannot start heartbeat link interface walk.");
 		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
@@ -593,7 +578,7 @@ static int
 clientstatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 {
 	/* Default value, its unit is milliseconds */
-	int timeout = 100;  
+	int timeout = 500;
 	const char * cstatus;
 	int ret = UNKNOWN_ERROR;
 
@@ -654,7 +639,7 @@ rscstatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 	if (FOR_HUMAN_READ == TRUE) {
 		printf("This node is holding %s resources.\n", rstatus);
 	} else {
-		printf("%s", rstatus);
+		printf("%s\n", rstatus);
 	}
 	return 0;
 }
@@ -697,7 +682,7 @@ hbparameter(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 
 	if ( paramname != NULL ) {
 		char * pvalue;
-		cl_log(LOG_INFO,"paramname: %s", paramname);
+
 		pvalue = hb->llc_ops->get_parameter(hb, paramname);
 		if (pvalue == NULL) {
 			cl_log(LOG_ERR, "Cannot get parameter %s's value", 
@@ -715,7 +700,7 @@ hbparameter(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		}
 		g_free(paramname);
 	} else {
-		cl_log(LOG_ERR, "Need parameter's name");
+		cl_log(LOG_ERR, "Missing parameter name");
 		ret_value = PARAMETER_ERROR;
 	}
 
