@@ -1,4 +1,4 @@
-/* $Id: hb_api.c,v 1.94.2.3 2004/09/11 20:52:32 alan Exp $ */
+/* $Id: hb_api.c,v 1.94.2.5 2005/01/03 20:56:08 gshi Exp $ */
 /*
  * hb_api: Server-side heartbeat API code
  *
@@ -1064,8 +1064,10 @@ api_send_client_msg(client_proc_t* client, struct ha_msg *msg)
 	}
 
 	if (CL_KILL(client->pid, client->signal) < 0 && errno == ESRCH) {
-		cl_log(LOG_INFO, "api_send_client: client %ld died"
-		,	(long) client->pid);
+		if (ANYDEBUG) {
+			cl_log(LOG_DEBUG, "api_send_client: client %ld died"
+			,	(long) client->pid);
+		}
 		if (!client->removereason) {
 			client->removereason = "died";
 		}
@@ -1121,9 +1123,6 @@ api_remove_client_int(client_proc_t* req, const char * reason)
 	client_proc_t*	prev = NULL;
 	client_proc_t*	client;
 
-		
-	api_send_client_status(req, LEAVESTATUS, reason);
-
 	--total_client_count;
 
 	if ((req->desired_types & DEBUGTREATMENTS) != 0) {
@@ -1147,23 +1146,31 @@ api_remove_client_int(client_proc_t* req, const char * reason)
 				prev->next = client->next;
 			}
 
-
 #if MAKEITCRASH
 			/* Drop the source - that will destroy the 'chan' */
 			if (client->gsource) {
 				G_main_del_IPC_Channel(client->gsource);
 			}
 #endif
-
-			/* Zap! */
-			memset(client, 0, sizeof(*client));
-			ha_free(client); client = NULL;
-			return;
+			break;
 		}
 		prev = client;
 	}
-	cl_log(LOG_ERR,	"api_remove_client_int: could not find pid [%ld]"
-	,	(long) req->pid);
+
+       if (req == client){
+               
+               api_send_client_status(req, LEAVESTATUS, reason);
+               
+               /* Zap! */
+               memset(client, 0, sizeof(*client));
+               ha_free(client); client = NULL;
+       }else{
+               cl_log(LOG_ERR, "api_remove_client_int: could not find pid [%ld]"
+                      ,        (long) req->pid);
+       }
+       
+       return;
+
 }
 
 
@@ -1441,9 +1448,11 @@ ProcessAnAPIRequest(client_proc_t*	client)
 
 		/* EOF? */
 		if (client->chan->ch_status == IPC_DISCONNECT) {
-			cl_log(LOG_INFO
-			,	"EOF from client pid %ld"
-			,	(long)client->pid);
+			if (ANYDEBUG) {
+				cl_log(LOG_DEBUG
+				,	"EOF from client pid %ld"
+				,	(long)client->pid);
+			}
 			client->removereason = "EOF";
 			goto getout;
 		}
@@ -1485,9 +1494,11 @@ getout:
 	/* May have gotten a message from 'client' */
 	if (CL_KILL(client->pid, 0) < 0 && errno == ESRCH) {
 		/* Oops... he's dead */
-		cl_log(LOG_INFO
-		,	"Client pid %ld died (input)"
-		,	(long)client->pid);
+		if (ANYDEBUG) {
+			cl_log(LOG_DEBUG
+			,	"Client pid %ld died (input)"
+			,	(long)client->pid);
+		}
 		client->removereason = "died";
 	}
 	if (DEBUGDETAILS) {
