@@ -1,4 +1,4 @@
-/* $Id: heartbeat_private.h,v 1.6.2.1 2004/04/20 05:05:28 alan Exp $ */
+/* $Id: heartbeat_private.h,v 1.6.2.4 2005/10/27 17:32:25 blaschke Exp $ */
 /*
  * heartbeat_private.h: definitions for the Linux-HA heartbeat program
  * that are defined in heartbeat.c and are used by other .c files
@@ -31,9 +31,11 @@
 #ifndef _HEARTBEAT_PRIVATE_H
 #define _HEARTBEAT_PRIVATE_H
 
+#include <heartbeat.h>
 #include <ha_msg.h>
 #include <glib.h>
 
+#include <clplumbing/longclock.h>
 #include <clplumbing/proctrack.h>
 #include <hb_proc.h>
 
@@ -48,7 +50,16 @@ void hb_init_watchdog(void);
 void hb_tickle_watchdog(void);
 void hb_close_watchdog(void);
 
+/* Used to register with heartbeat for receiving messages directly */
+typedef void (*HBmsgcallback) (const char * type, struct node_info* fromnode
+,	TIME_T msgtime, seqno_t seqno, const char * iface, struct ha_msg * msg);
+void hb_register_msg_callback(const char * msgtype, HBmsgcallback callback);
+void hb_register_comm_up_callback(void(*callback)(void));
+
+
 int  hb_send_resources_held(int stable, const char * comment);
+void hb_setup_child(void);
+void init_resource_module(void);
 
 gboolean hb_mcp_final_shutdown(gpointer p);
 gboolean hb_send_local_status(gpointer p);
@@ -62,6 +73,7 @@ void hb_versioninfo(void);
 void hb_dump_proc_stats(volatile struct process_info * proc);
 void hb_trigger_restart(int quickrestart);
 
+void hb_shutdown_if_needed(void);
 void hb_giveup_resources(void);
 void hb_kill_tracked_process(ProcTrack* p, void * data);
 
