@@ -1,4 +1,4 @@
-/* $Id: md5.c,v 1.9.2.1 2004/04/20 09:08:50 lars Exp $ */
+/* $Id: md5.c,v 1.9.2.2 2005/01/12 02:25:46 horms Exp $ */
 /*
  * This code implements the MD5 message-digest algorithm.
  * The algorithm is due to Ron Rivest.  This code was
@@ -365,10 +365,10 @@ md5_auth_calc(const struct HBauth_info *t, const void * text
 	if (key_len > MD5_BLOCKSIZE) { 
 		MD5Context      tctx;   
 		MD5Init(&tctx);
-		MD5Update(&tctx, key, key_len);
+		MD5Update(&tctx, (const unsigned char*) key, key_len);
 		MD5Final(tk, &tctx); 
 
-		key = tk;
+		key = (char*) tk;
 		key_len = MD5_DIGESTSIZE;
 	}       
 	/* start out by storing key in pads */
