@@ -1,4 +1,4 @@
-/* $Id: apcsmart.c,v 1.12.2.1 2004/04/20 08:50:06 alan Exp $ */
+/* $Id: apcsmart.c,v 1.12.2.4 2005/04/22 13:53:26 blaschke Exp $ */
 /*
  * Stonith module for APCSmart Stonith device
  * Copyright (c) 2000 Andreas Piesk <a.piesk@gmx.net>
@@ -42,18 +42,17 @@
 #include <glib.h>
 
 /*
- * APCSmart (tested with 2 old 900XLI)
+ * APCSmart (tested with old 900XLI, APC SmartUPS 700 and SmartUPS-1000)
  *
- * no configfile needed. every host performing resets using
- * this module must be connected to a ups via /dev/ups,
- * /dev/ups is a symlink to the proper tty device.
- *
- * the reset is a combined reset (cmd: S@000).
- * that means if the ups is online, a scheduled reset (20s delay)
- * will be triggered. after the reset the ups will immediately
- * return  online. if the ups is on-battery, the reset will also be
- * a scheduled reset but the ups will remain offline until the power
- * is back. 
+ * The reset is a combined reset: "S" and "@000"
+ * The "S" command tells the ups that if it is on-battery, it should
+ * remain offline until the power is back. 
+ * If that command is not accepted, the "@000" command will be sent
+ * to tell the ups to turn off and back on right away.
+ * In both cases, if the UPS supports a 20 second shutdown grace
+ * period (such as on the 900XLI), the shutdown will delay that long,
+ * otherwise the shutdown will happen immediately (the code searches
+ * for the smallest possible delay).
  */
 
 #define	DEVICE	                "APCSmart-Stonith"
@@ -66,8 +65,7 @@
 #define SEND_DELAY		50000	/* in s */
 #define ENDCHAR			10	/* use LF */
 #define MAX_STRING              512
-#define SHUTDOWN_DELAY		"020"
-#define WAKEUP_DELAY		"000"
+#define MAX_DELAY_STRING	16
 #define SWITCH_TO_NEXT_VAL	"-"	/* APC cmd for cycling through
 					 * the values
 					 */
@@ -76,9 +74,10 @@
 #define RSP_SMART_MODE		"SM"
 #define CMD_GET_STATUS		"Q"
 #define RSP_GET_STATUS		NULL
-#define CMD_RESET               "S@000"
-#define RSP_RESET		"*"
-#define RSP_RESET2		"OK"
+#define CMD_RESET               "S"	/* turn off & stay off if on battery */
+#define CMD_RESET2              "@000"	/* turn off & immediately turn on */
+#define RSP_RESET		"*"	/* RESET response from older models */
+#define RSP_RESET2		"OK"	/* RESET response from newer models */
 #define CMD_SHUTDOWN_DELAY	"p"
 #define CMD_WAKEUP_DELAY	"r"
 
@@ -90,12 +89,14 @@ struct APCDevice {
     int hostcount;		/* of hosts (1)				*/
     char *upsdev;		/*					*/
     int upsfd;			/* for serial port			*/
+    char shutdown_delay[MAX_DELAY_STRING]; /* smallest shutdown delay   */
+    char old_shutdown_delay[MAX_DELAY_STRING]; /* old shutdown delay	*/
+    char wakeup_delay[MAX_DELAY_STRING]; /* smallest wakeup delay	*/
+    char old_wakeup_delay[MAX_DELAY_STRING]; /* old wakeup delay	*/
 };
 
 /* saving old settings */
 static struct termios old_tio;
-static char old_shutdown_delay[MAX_STRING];
-static char old_wakeup_delay[MAX_STRING];
 
 static int f_serialtimeout;	/* flag for timeout */
 static const char *APCid = DEVICE;
@@ -224,9 +225,10 @@ int APC_send_cmd(int upsfd, const char *cmd);
 int APC_recv_rsp(int upsfd, char *rsp);
 int APC_enter_smartmode(int upsfd);
 int APC_set_ups_var(int upsfd, const char *cmd, char *newval);
+int APC_get_smallest_delay(int upsfd, const char *cmd, char *smdelay);
 int APC_parse_config_info(struct APCDevice *ad, const char *info );
 int APC_init( struct APCDevice *ad );
-void APC_deinit( int upsfd );
+void APC_deinit( struct APCDevice *ad );
 
 /*
  *
@@ -342,7 +344,7 @@ APC_open_serialport(const char *port, speed_t speed)
 #ifdef APC_DEBUG
 	syslog(LOG_DEBUG, "%s: 1st open failed.", __FUNCTION__);
 #endif
-	return (f_serialtimeout ? S_TIMEOUT : S_OOPS);
+	return -1;
     }
 
     if (file_lock(fd) != 0) {
@@ -350,7 +352,8 @@ APC_open_serialport(const char *port, speed_t speed)
 #ifdef APC_DEBUG
 	syslog(LOG_DEBUG, "%s: 1st lock failed.", __FUNCTION__);
 #endif
-	return (S_OOPS);
+	close(fd);
+	return -1;
     }
 
     tcgetattr(fd, &old_tio);
@@ -384,7 +387,7 @@ APC_open_serialport(const char *port, speed_t speed)
 #ifdef APC_DEBUG
 	syslog(LOG_DEBUG, "%s: 2nd open failed.", __FUNCTION__);
 #endif
-	return (f_serialtimeout ? S_TIMEOUT : S_OOPS);
+	return -1;
     }
 
     if (file_lock(fd) != 0) {
@@ -393,7 +396,8 @@ APC_open_serialport(const char *port, speed_t speed)
 	syslog(LOG_DEBUG, "%s: 2nd lock failed.", __FUNCTION__);
 #endif
 
-	return (f_serialtimeout ? S_TIMEOUT : S_OOPS);
+	close(fd);
+	return -1;
     }
 
     tcgetattr(fd, &tio);
@@ -558,6 +562,10 @@ APC_set_ups_var(int upsfd, const char *cmd, char *newval)
 	((rc = APC_recv_rsp(upsfd, orig)) != S_OK))
 	return (rc);
 
+#ifdef APC_DEBUG
+    syslog(LOG_DEBUG, "%s: var '%s' original val %s", __FUNCTION__, cmd, orig);
+#endif
+
     if (strcmp(orig, newval) == 0)
 	return (S_OK);		/* already set */
 
@@ -574,16 +582,73 @@ APC_set_ups_var(int upsfd, const char *cmd, char *newval)
 	    return (rc);
 
 	if (strcmp(resp, newval) == 0) {
+#ifdef APC_DEBUG
+	    syslog(LOG_DEBUG, "%s: var '%s' set to %s", __FUNCTION__, 
+		    						cmd, newval);
+#endif
 	    strcpy(newval, orig);	/* return the old value */
 	    return (S_OK);		/* got it */
 	}
     }
 
-    syslog(LOG_ERR, "%s: variable '%s' wrapped!", __FUNCTION__, cmd);
-
+    syslog(LOG_ERR, "%s(): variable '%s' wrapped!", __FUNCTION__, cmd);
+    syslog(LOG_ERR, "%s(): This UPS may not support STONITH :-("
+    ,	 __FUNCTION__);
     return (S_OOPS);
 }
 
+/* 
+ * Query the smallest delay supported by the hardware using the 
+ * <cmdchar> '-' (repeat) approach and looping through all possible values,
+ * saving the smallest
+ */
+
+int
+APC_get_smallest_delay(int upsfd, const char *cmd, char *smdelay)
+{
+	char resp[MAX_DELAY_STRING];
+	char orig[MAX_DELAY_STRING];
+	int delay, smallest;
+	int rc;
+
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s: called.", __FUNCTION__);
+	}
+
+	if (((rc = APC_enter_smartmode(upsfd)) != S_OK)
+	||	((rc = APC_send_cmd(upsfd, cmd)) != S_OK)
+	||	((rc = APC_recv_rsp(upsfd, orig)) != S_OK)) {
+			return (rc);
+	}
+
+	smallest = atoi(orig);
+	strcpy(smdelay, orig);
+
+	*resp = '\0';
+
+	/* search for smallest delay; need to loop through all possible
+	 * values so that we leave delay the way we found it */
+	while (strcmp(resp, orig) != 0) {
+		if (((rc = APC_send_cmd(upsfd, SWITCH_TO_NEXT_VAL)) != S_OK)
+		||	((rc = APC_recv_rsp(upsfd, resp)) != S_OK)) {
+	    			return (rc);
+		}
+
+		if (((rc = APC_enter_smartmode(upsfd)) != S_OK)
+		||	((rc = APC_send_cmd(upsfd, cmd)) != S_OK)
+		||	((rc = APC_recv_rsp(upsfd, resp)) != S_OK)) {
+	    			return (rc);
+		}
+
+		if ((delay = atoi(resp)) < smallest) {
+			smallest = delay;
+			strcpy(smdelay, resp);
+		}
+	}
+
+	return (S_OK);
+}
+
 /*
  * Initialize the ups
  */
@@ -592,35 +657,57 @@ int
 APC_init( struct APCDevice *ad )
 {
   int upsfd;
-  char value[MAX_STRING];
+  char value[MAX_DELAY_STRING];
 
 #ifdef APC_DEBUG
   syslog(LOG_DEBUG, "%s: called.", __FUNCTION__);
 #endif
 
-  /* if ad->upsfd == -1 -> dev configured! */
-  if(ad->upsfd != -1 ) return( S_OK );
+  /* if ad->upsfd != -1 device has already been configured. */
+  /* Just enter smart mode again because otherwise a SmartUPS-1000 */
+  /*   has been observed to sometimes not respond. */
+  if(ad->upsfd != -1) {
+      if(APC_enter_smartmode(ad->upsfd) != S_OK)
+	  return(S_OOPS);
+      return( S_OK );
+  }
 
   /* open serial port and store the fd in ad->upsfd */
-  if ((upsfd = APC_open_serialport(ad->upsdev, B2400)) == -1)
-    return (-1);
+  if ((upsfd = APC_open_serialport(ad->upsdev, B2400)) == -1) {
+    return (S_OOPS);
+  }
 
   /* switch into smart mode */
-  if(APC_enter_smartmode(upsfd) != S_OK)
-    return( -1 );
+  if(APC_enter_smartmode(upsfd) != S_OK) {
+    APC_close_serialport(upsfd);
+    return( S_OOPS );
+  }
+
+  /* get the smallest possible delays for this particular hardware */
+  if (APC_get_smallest_delay(upsfd, CMD_SHUTDOWN_DELAY
+    , ad->shutdown_delay) != S_OK
+  || APC_get_smallest_delay(upsfd, CMD_WAKEUP_DELAY
+    , ad->wakeup_delay) != S_OK) {
+    APC_close_serialport(upsfd);
+    return S_OOPS;
+  }
 
   /* get the old settings and store them */
-  strcpy(value, SHUTDOWN_DELAY);
-  if (APC_set_ups_var(upsfd, CMD_SHUTDOWN_DELAY, value) != S_OK)
-    return (-1);  
+  strcpy(value, ad->shutdown_delay);
+  if (APC_set_ups_var(upsfd, CMD_SHUTDOWN_DELAY, value) != S_OK) {
+    APC_close_serialport(upsfd);
+    return (S_OOPS);  
+  }
 
-  strcpy(old_shutdown_delay, value);
+  strcpy(ad->old_shutdown_delay, value);
 
-  strcpy(value, WAKEUP_DELAY);
-  if (APC_set_ups_var(upsfd, CMD_WAKEUP_DELAY, value) != S_OK)
-    return (-1);
+  strcpy(value, ad->wakeup_delay);
+  if (APC_set_ups_var(upsfd, CMD_WAKEUP_DELAY, value) != S_OK) {
+    APC_close_serialport(upsfd);
+    return (S_OOPS);
+  }
 
-  strcpy(old_wakeup_delay, value);
+  strcpy(ad->old_wakeup_delay, value);
 
   ad->upsfd = upsfd;
 
@@ -632,15 +719,15 @@ APC_init( struct APCDevice *ad )
  */
 
 void
-APC_deinit( int upsfd )
+APC_deinit( struct APCDevice *ad )
 {
-      APC_enter_smartmode( upsfd );
+      APC_enter_smartmode( ad->upsfd );
 
-      APC_set_ups_var(upsfd, CMD_SHUTDOWN_DELAY, old_shutdown_delay);
-      APC_set_ups_var(upsfd, CMD_WAKEUP_DELAY, old_wakeup_delay);
+      APC_set_ups_var(ad->upsfd, CMD_SHUTDOWN_DELAY, ad->old_shutdown_delay);
+      APC_set_ups_var(ad->upsfd, CMD_WAKEUP_DELAY, ad->old_wakeup_delay);
 
       /* close serial port */
-      APC_close_serialport(upsfd);
+      APC_close_serialport(ad->upsfd);
 }
 
 /*
@@ -720,13 +807,13 @@ apcsmart_status(Stonith * s)
     rc = APC_init(ad);
 
     /* get status */
-    if (((rc = APC_init( ad ) == S_OK) &&
+    if (((rc = APC_init( ad )) == S_OK) &&
 	((rc = APC_send_cmd(ad->upsfd, CMD_GET_STATUS)) == S_OK) &&
-	((rc = APC_recv_rsp(ad->upsfd, resp)) == S_OK)))
+	((rc = APC_recv_rsp(ad->upsfd, resp)) == S_OK))
 	return (S_OK);		/* everything ok. */
 
 #ifdef APC_DEBUG
-    syslog(LOG_DEBUG, "%s: failed.", __FUNCTION__);
+    syslog(LOG_DEBUG, "%s: failed, rc=%d.", __FUNCTION__, rc);
 #endif
 
     return (rc);
@@ -806,6 +893,36 @@ apcsmart_free_hostlist(char **hlist)
     hlist = NULL;
 }
 
+static gboolean
+apcsmart_RegisterBitsSet(struct APCDevice * ad, int nreg, unsigned bits
+,	gboolean* waserr)
+{
+	const char*	reqregs[4] = {"?", "~", "'", "8"};
+	unsigned	regval;
+	char		resp[MAX_STRING];
+
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s: called.", __FUNCTION__);
+	}
+
+
+	if (APC_enter_smartmode(ad->upsfd) != S_OK
+	||	APC_send_cmd(ad->upsfd, reqregs[nreg]) != S_OK
+	||	APC_recv_rsp(ad->upsfd, resp) != S_OK
+	||	(sscanf(resp, "%02x", &regval) != 1)) {
+		if (waserr){
+			*waserr = TRUE;
+		}
+		return FALSE;
+	}
+	if (waserr){
+		*waserr = FALSE;
+	}
+	return ((regval & bits) == bits);
+}
+
+#define	apcsmart_ResetHappening(ad,err) apcsmart_RegisterBitsSet(ad,3,0x08,err)
+
 /*
  * reset the host 
  */
@@ -866,29 +983,37 @@ apcsmart_reset_req(Stonith * s, int request, const char *host)
       goto out;
     }
 
-    /* enter smartmode, send reset command */
+    /* send reset command(s) */
     if (((rc = APC_init(ad)) == S_OK)
-	&& ((rc = APC_send_cmd(ad->upsfd, CMD_RESET)) == S_OK)
-	&& ((rc = APC_recv_rsp(ad->upsfd, resp)) == S_OK)
-	&& (strcmp(resp, RSP_RESET) == 0 || strcmp(resp, RSP_RESET2) == 0)) {
+        && ((rc = APC_send_cmd(ad->upsfd, CMD_RESET)) == S_OK)) {
+	if (((rc = APC_recv_rsp(ad->upsfd, resp)) == S_OK) &&
+	    (strcmp(resp, RSP_RESET) == 0 || strcmp(resp, RSP_RESET2) == 0)) {
+	    /* first kind of reset command was accepted */
+	} else if (((rc = APC_send_cmd(ad->upsfd, CMD_RESET2)) == S_OK)
+	    && ((rc = APC_recv_rsp(ad->upsfd, resp)) == S_OK) &&
+	    (strcmp(resp, RSP_RESET) == 0 || strcmp(resp, RSP_RESET2) == 0)) {
+	    /* second kind of command was accepted */
+	} else {
+	    rc = S_RESETFAIL;
+	}
+    }
+    if (rc == S_OK) {
+	/* we wait grace period + up to 10 seconds after shutdown */
+	int	maxdelay = atoi(ad->shutdown_delay)+10;
 
-	/* ok, reset is initiated. ups don't accept any cmds until */
-	/* reboot -> reboot complete if status cmd accepted */
-	/* we wait max. 30 sec after shutdown */
-	/* (shutdown delay + 10 seconds) */
-
-	sleep(atoi(SHUTDOWN_DELAY));
-
-	/* ups should be dead now -> wait for rebirth */
-
-	for (i = 0; i < 10; i++) {
-	    if (((rc = APC_send_cmd(ad->upsfd, CMD_GET_STATUS)) == S_OK) &&
-	        ((rc = APC_recv_rsp(ad->upsfd, resp)) == S_OK)) {
-		    rc = S_OK;
-		    goto out;
+	for (i=0; i < maxdelay; ++i) {
+	    gboolean	err;
+	    if (apcsmart_ResetHappening(ad, &err)) {
+	        if (err) {
+	            break;
+	        }else{
+	            rc = S_OK;
+	            goto out;
+	        }
 	    }
 	    sleep(1);
 	}
+	rc = S_RESETFAIL;
     }
 
     /* reset failed */
@@ -990,19 +1115,19 @@ apcsmart_getinfo(Stonith * s, int reqtype)
 		break;
 
     	case ST_CONF_INFO_SYNTAX:
-        	ret = _("hostname devicename\n"
+        	ret = _("devicename hostname\n"
                 	"The hostname and devicename are white-space delimited.");
         	break;
 
     	case ST_CONF_FILE_SYNTAX:
-        	ret = _("hostname devicename\n"
+        	ret = _("devicename hostname\n"
                 	"The hostname and devicename are white-space delimited.\n"
-                	"All three items must be on one line.\n"
+                	"Both items must be on one line.\n"
                 	"Blank lines and lines beginning with # are ignored.");
 		break;
 
 	case ST_DEVICEDESCR:
-		ret = _("APC Smart UPS (via serial port)");
+		ret = _("APC Smart UPS (via serial port - model must be >= Smart-UPS 750)");
 		break;
 
 	case ST_DEVICEURL:
@@ -1038,7 +1163,7 @@ apcsmart_destroy(Stonith * s)
 
     ad = (struct APCDevice *) s->pinfo;
 
-    APC_deinit( ad->upsfd );
+    APC_deinit( ad );
 
     ad->APCid = NOTapcID;
 
